function [num_true_positive_matrix, num_potentially_false_negative_matrix, num_potentially_false_positive_matrix, num_Gamma_I_matrix, success_rate, num_samples_matrix ] = test_alias1ri_hc_weightedhc_multiple(Gamma, index_set_I, c, L_array, num_repeat)
%TEST_ALIAS1RI_HC_WEIGHTED_HC_MULTIPLE Performs tests from Section 4.1.3.
%
%   Input parameters:
%      Gamma - Index set of frequency candidates
%      index_set_I - true frequencies
%      c - parameter
%      L_array - values for number of lattices in rank-1 lattice configuration
%      num_repeat - number of times to perform tests
%
%   Output parameters:
%      num_true_positive_matrix - true positives, matrix of
%                                 size length(L_array) x num_repeat
%      num_potentially_false_negative_matrix - potentially false negatives,
%                                 size length(L_array) x num_repeat
%      num_potentially_false_positive_matrix - potentially false positives,
%                                 size length(L_array) x num_repeat
%      num_Gamma_I_matrix - number of detected frequencies
%                                 size length(L_array) x num_repeat
%      success_rate - success rates, size length(L_array) x num_repeat
%      num_samples_matrix - number of samples, size length(L_array) x num_repeat
%
% Copyright (c) 2020 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

d = size(Gamma, 2);
num_L = numel(L_array);
sparsity = size(index_set_I,1);

lambda = max(max(max(Gamma,[],1)-min(Gamma,[],1)), floor(c*sparsity));
M = simplenextprime(lambda);

num_true_positive_matrix = nan(num_L, num_repeat);
num_potentially_false_negative_matrix = nan(num_L, num_repeat);
num_potentially_false_positive_matrix = nan(num_L, num_repeat);
num_Gamma_I_matrix = nan(num_L, num_repeat);
num_samples_matrix = nan(num_L, num_repeat);

for iL=1:num_L
  L = L_array(iL);

  for irepeat = 1:num_repeat
    if mod(irepeat,ceil(num_repeat/100)) == 0; fprintf('hc/weightedhc: L=%d, irepeat=%d\n', L, irepeat); end
    M_array = repmat(M, 1, L);
    z_matrix = randi(M, L, d) - 1;
    
    [num_true_positive, num_potentially_false_negative, num_potentially_false_positive, num_Gamma_I, num_samples] = test_alias1ri_run(Gamma, index_set_I, z_matrix, M_array);
    
    num_true_positive_matrix(iL, irepeat) = num_true_positive;
    num_potentially_false_negative_matrix(iL, irepeat) = num_potentially_false_negative;
    num_potentially_false_positive_matrix(iL, irepeat) = num_potentially_false_positive;
    num_Gamma_I_matrix(iL, irepeat) = num_Gamma_I;
    num_samples_matrix(iL, irepeat) = num_samples;
  end
end

success_rate = sum(num_true_positive_matrix == num_Gamma_I_matrix, 2) / num_repeat;

end

