function [num_true_positive, num_potentially_false_negative, num_potentially_false_positive, num_I_tilde, num_samples] = test_alias1ri_run(Gamma, index_set_I, z_matrix, M_array)
%TEST_ALIAS1RI_RUN Runs single test for examples in Section 4.1.2 and 4.1.3.
%
%   Input parameters:
%      Gamma - Index set of frequency candidates
%      index_set_I - sparse frequency index set
%      z_matrix - generating vectors
%      M_array - lattices sizes
%
%   Output parameters:
%      num_true_positive - true positives
%      num_potentially_false_negative - potentially false negatives
%      num_potentially_false_positive - potentially false positives
%      num_I_tilde - number of detected frequencies
%      num_samples - number of samples
%
% Copyright (c) 2020 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

tau = 1e-6;

L = length(M_array);

samples = cell(1,L);

for ell = 1:L
  samples{ell} = trig_poly_eval(index_set_I, ones(size(index_set_I,1),1), {z_matrix(ell,:), M_array(ell),[]});
end

[I_tilde, p_hat_tilde] = sfft_random_r1l_alg2ri(Gamma, z_matrix, M_array, samples, tau);

ind_I_tilde_eq1 = find(abs(p_hat_tilde-1) <= tau);
% ind_I_tilde_eq0 = find(abs(p_hat_tilde) <= tau);
ind_I_tilde_ge2 = find((real(p_hat_tilde) >= 2-tau) .* (abs(imag(p_hat_tilde)) <= tau));
ind_I_tilde_classified = union(ind_I_tilde_eq1, ind_I_tilde_ge2);
ind_I_tilde_ge1 = find((real(p_hat_tilde) >= 1-tau) .* (abs(imag(p_hat_tilde)) <= tau));

if length(ind_I_tilde_classified) ~= size(I_tilde,1)
  error('unclassified indices');
end

I_tilde_eq1 = I_tilde(ind_I_tilde_eq1, :);
[~, ind] = intersect(I_tilde_eq1, index_set_I, 'rows');
num_true_positive = length(ind);

if ~isempty(ind_I_tilde_ge2)
  I_tilde_ge2 = I_tilde(ind_I_tilde_ge2, :);
  [~, ind] = intersect(I_tilde_ge2, index_set_I, 'rows');
  num_potentially_false_negative = length(ind);
else
  num_potentially_false_negative = 0;
end

if ~isempty(ind_I_tilde_ge1)
  I_tilde_ge1 = I_tilde(ind_I_tilde_ge1, :);
  [~, ind] = setdiff(I_tilde_ge1, index_set_I, 'rows');
  num_potentially_false_positive = length(ind);
else
  num_potentially_false_positive = 0;
end

num_I_tilde = size(I_tilde,1);

num_samples = sum(M_array) - L + 1;

end

